Package: gapclosing
Title: Estimate Gaps Under an Intervention
Version: 1.0.3
Authors@R: 
    person(given = "Ian",
           family = "Lundberg",
           role = c("aut", "cre"),
           email = "ianlundberg@ucla.edu",
           comment = c(ORCID = "https://orcid.org/0000-0002-1909-2270"))
Description: Provides functions to estimate the disparities across categories (e.g. Black and white) that persists if a treatment variable (e.g. college) is equalized. Makes estimates by treatment modeling, outcome modeling, and doubly-robust augmented inverse probability weighting estimation, with standard errors calculated by a nonparametric bootstrap. Cross-fitting is supported. Survey weights are supported for point estimation but not for standard error estimation; those applying this package with complex survey samples should consult the data distributor to select an appropriate approach for standard error construction, which may involve calling the functions repeatedly for many sets of replicate weights provided by the data distributor. The methods in this package are described in the accompanying paper: <doi:10.1177/00491241211055769>.
License: MIT + file LICENSE
URL: https://ilundberg.github.io/gapclosing/
BugReports: https://github.com/ilundberg/gapclosing/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: stats, mgcv, ranger, glmnet, magrittr, dplyr, forcats,
        ggplot2, tidyr, foreach, tidyselect, parallel, doParallel,
        doRNG
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, spelling
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-22 23:36:28 UTC; ilundberg
Author: Ian Lundberg [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1909-2270>)
Maintainer: Ian Lundberg <ianlundberg@ucla.edu>
Config/testthat/edition: 3
Language: en-US
Repository: CRAN
Date/Publication: 2026-05-23 11:00:20 UTC
Built: R 4.5.2; ; 2026-05-25 02:18:07 UTC; unix
