# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,gapclosing)
S3method(plot,gapclosing)
S3method(print,gapclosing)
S3method(summary,gapclosing)
export("%>%")
export(cross_fit_estimator)
export(df_to_gapclosing_list)
export(disparityplot)
export(fit_ridge)
export(gapclosing)
export(generate_simulated_data)
export(pairwise_diff)
export(point_estimator)
importFrom(doRNG,"%dorng%")
importFrom(foreach,"%do%")
importFrom(magrittr,"%>%")
